% clear old variables:
clear variables;
% use numerical integration and plot 
% steering for bicycle model of car
% init conds [theta ya xb yb]

%--------------------------------------
% Constants
%--------------------------------------

L = 0.3;     % car length m
V = 3.0;
theta = -0.0; % parallel to line
% specify position of front of car:
xa = -0.0; ya = 0.1;
xb = xa -L * cos(theta); yb = ya + L * sin(theta); 
y0 = [theta ya xb yb 0]; %initial conditions
t0 = 0.0;   tf = 12.0;    % simulate for 12.0 seconds

%Make uniform time sampling
tspan=(1:1000)/1000*tf;

[t,y] = ode45('pidtrack_2011',tspan,y0);
y1=y(:,1);  % theta: orientation wrt line
y2=y(:,2);  % ya: offset wrt line of front of car
y3=y(:,3);  % xb: longitudinal position of back of car
y4=y(:,4);  % yb: lateral position of back of car
y5=y(:,5);  % delta: steering angle


xbdot = V*cos(y1);
ybdot =	-V*sin(y1);

xa = y3 + L * cos(y1);



%--------------------------------------
% Plot track
%--------------------------------------
xTrack = [];
yTrack = [];

f1 = @(x,y) -(sqrt((x)^2+(y-(-1.7))^2) - 1.7);
f2 = @(x,y) -y;
f3 = @(x,y) -y + 0.1;
f4 = @(x,y) -(sqrt((x-6)^2+(y-(-1))^2) - 1.1);
f5 = @(x,y) -x + 7.1;
f6 = @(x,y) (sqrt((x-5.8)^2+(y-(-5.4))^2) - 1.3);
f7 = @(x,y) -(sqrt((x-2.5)^2+(y-(-5.4))^2) - 2);
f8 = @(x,y) y + 3.4;

trackSegments = cell(2,8);
trackSegments(:,1) = {f1;[-3 0 -4 1]};
trackSegments(:,2) = {f2;[0 3 -1 1]};
trackSegments(:,3) = {f3;[3 6 -1 1]};
trackSegments(:,4) = {f4;[6 9 1 -1]};
trackSegments(:,5) = {f5;[6 9 -4 -1]};
trackSegments(:,6) = {f6;[4.5 9 -8 -5.4]};
trackSegments(:,7) = {f7;[4.5 2.5 -3.4 -5.4]};
trackSegments(:,8) = {f8;[2.5 0 -1 -3.4]};

h = gcf;
for i = 1:size(trackSegments,2)
    h1 = ezplot(trackSegments{1,i},trackSegments{2,i},h);
%     pause;
    if i == 4
        xTrack = [xTrack fliplr(get(h1,'xdata'))];
    else
        xTrack = [xTrack get(h1,'xdata')];
    end
     if i == 3 || i == 3 || i == 5 || i == 6 || i == 7
         yTrack = [yTrack get(h1,'ydata')];
     else
         yTrack = [yTrack fliplr(get(h1,'ydata'))];
     end
end

%--------------------------------------
%Plot information
%--------------------------------------

for i = 1:length(xa)
	yadot = -V*(sin(y1(i)) + tan(y5(i))*cos(y1(i)));
	xadot =  V*(cos(y1(i)) + tan(y5(i))*sin(y1(i)));
	errout = track(xa(i),y2(i),xadot,yadot);
    e(i) = errout(1);
    edot(i) = errout(2);
end

% figure();
subplot(2,2,1), plot(xa, y2, 'r', xTrack, yTrack, 'b'), grid on, ylabel('y position'),
xlabel('x position');
% axis([-1.5 6 -2.5 0.5]);
axis equal;

subplot(2,2,2), plot(t, y5), grid on, ylabel('delta'),
xlabel('time');

subplot(2,2,3), plot(t, e, t, edot), grid on, ylabel('lateral error, edot'),
xlabel('time');

%subplot(2,2,4), plot(t, y1), grid on, ylabel('theta'),
%xlabel('time');

subplot(2,2,4), plot(t, xa, t, y2), grid on, ylabel('xa,ya'),
xlabel('time');



% write results to a file
 fid=fopen('circ-out.txt','w');
 fprintf(fid,'# t theta ya delta \n');
 temp=[t';y1';y2';y5'];   % vectorize 
 fprintf(fid,'%8.4f %8.4f %8.4f %8.4f\n', temp);
 fid=fopen('circ-xy.txt','w');
 temp=[xa'; y2'];   % vectorize 
 fprintf(fid,'# xa ya \n');
 fprintf(fid,'%8.4f %8.4f \n', temp);


